﻿using CoreLibrary.Exceptions;
using CoreLibrary.Persistance;
using CoreLibrary.Evenements;
using CoreLibrary.Regles;
using CoreLibrary.Statistiques;
using System.Runtime.Serialization;
using CoreLibrary.Core;

namespace CoreLibrary.Joueurs
{
    /// <summary>
    /// Classe représentant un joueur.
    /// </summary>
    [DataContract]
    [KnownType(typeof(ReglesClassiques))]
    [KnownType(typeof(ReglesDifficiles))]
    public class Joueur : IEstPersistant
    {
        /// <summary>
        /// Événement déclenché lorsque le joueur se connecte.
        /// </summary>
        public event EventHandler<JoueurSeConnecterEventArgs>? JoueurSeConnecter;

        private void QuandJoueurSeConnecter(Joueur joueur) => JoueurSeConnecter?.Invoke(this, new JoueurSeConnecterEventArgs(joueur));
        
        [DataMember]
        private Dictionary<(IRegles, Statistique), int> statistiques = new Dictionary<(IRegles, Statistique), int>();

        /// <summary>
        /// Obtient le nom du joueur.
        /// </summary>
        [DataMember]
        public string Nom { get; private set; } = "";

        /// <summary>
        /// Indique si le joueur est connecté.
        /// </summary>
        public bool EstConnecte { get; private set; } = false;

        /// <summary>
        /// Constructeur de joueur.
        /// </summary>
        public Joueur()
        {
        }

        /// <summary>
        /// Constructeur de joueur avec le nom spécifié.
        /// </summary>
        /// <param name="nom">Le nom du joueur.</param>
        public Joueur(string nom)
        {
            Nom = nom;
        }

        public void SeConnecter(Joueur joueur)
        {
            if (EstConnecte)
                throw new JoueurDejaConnecteException(this);

            QuandJoueurSeConnecter(joueur);
        }

        public void JouerPartie(Partie partie)
        {
            partie.PartieDemanderJoueurJouer += QuandDemanderJoueurJouer;
        }

        public virtual void QuandDemanderJoueurJouer(object? sender, PartieDemanderJoueurJouerEventArgs e)
        {
            if (e.Nom != Nom)
                return;
        }

        /// <summary>
        /// Retourne une représentation sous forme de chaîne de caractères.
        /// </summary>
        /// <returns>Renvoie une chaîne de caractères du nom du joueur.</returns>
        public override string ToString() => Nom;

        public int Statistique(IRegles regles, Statistique statistique) => statistiques.GetValueOrDefault((regles, statistique),0);

        /// <summary>
        /// Incrémente la valeur de la statistique spécifiée pour les règles données.
        /// </summary>
        /// <param name="regles">Les règles du jeu.</param>
        /// <param name="statistique">Le type de statistique.</param>
        public void IncrementerStatistique(IRegles regles, Statistique statistique) => 
            statistiques[(regles, statistique)] = Statistique(regles, statistique) + 1;
    }
}
